//
//  GLAtlasKit.h
//  GLAtlasKit
//
//  Created by Jeffrey Early on 11/25/07.
//  Copyright 2007 Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>
#import <GLAtlasKit/GLReverseGeocoder.h>

extern NSString *GLAtlasKitDefaultReverseGeocoderDidChange;

extern NSString *GLAtlasKitDefaultReverseGeocoderDidChange;

@interface GLAtlasKit : NSObject {
	NSMutableArray *timezoneArray;
	
	NSArray *availableReverseGeocoders;
	id selectedReverseGeocoder;
	
	id selectedGeocoder;
	
    NSPersistentStoreCoordinator *persistentStoreCoordinator;
    NSManagedObjectModel *managedObjectModel;
    NSManagedObjectContext *managedObjectContext;

}

/************************************************/
/*		Initialization							*/
/************************************************/

#pragma mark -
#pragma mark Initialization
#pragma mark

+ (id) sharedAtlasKit;

/************************************************/
/*		Time Zone								*/
/************************************************/

#pragma mark -
#pragma mark Time Zone
#pragma mark

@property(strong) NSMutableArray *timezoneArray;

- (NSTimeZone *) timeZoneForPoint: (GLPoint) point;
- (NSTimeZone *) timeZoneForPointGeometry: (GLPointGeometry *) point;

/************************************************/
/*		Reverse Geocoding						*/
/************************************************/

#pragma mark -
#pragma mark Reverse Geocoding
#pragma mark

@property(readonly) NSArray *availableReverseGeocoders;
@property(nonatomic, strong) NSObject <GLReverseGeocoder> * selectedReverseGeocoder;
@property NSUInteger selectedReverseGeocoderIndex;

- (NSDictionary *) descriptionForLatitude: (double) latitude longitude: (double) longitude;
- (void) descriptionForLatitude: (double) latitude longitude: (double) longitude delegate: (id) aDelegate didFinishSelector: (SEL) selector;

// Decodes the dictionary, returning a human-readable string describing the location.
- (NSString *) locationDescriptionFromDictionary: (NSDictionary *) dictionary;

/************************************************/
/*		Geocoding								*/
/************************************************/

#pragma mark -
#pragma mark Geocoding
#pragma mark

// Returns a single point, representing the address
// Synchronous request and asynchronous request -- the return selector should take one argument, the return point.
- (GLPoint) latitudeLongitudeForAddress: (NSString *) address;
- (void) latitudeLongitudeForAddress: (NSString *) address delegate: (id) aDelegate didFinishSelector: (SEL) selector;

// Returns a rect bounding the address (rect may zero size)
// Synchronous request and asynchronous request -- the return selector should take one argument, the return rect.
- (GLRect) boundingRectForAddress: (NSString *) address;
- (void) boundingRectForAddress: (NSString *) address delegate: (id) aDelegate didFinishSelector: (SEL) selector;

/************************************************/
/*		CoreData								*/
/************************************************/

#pragma mark -
#pragma mark CoreData
#pragma mark

//@property(readonly) NSPersistentStoreCoordinator *persistentStoreCoordinator;
//@property(readonly) NSManagedObjectModel *managedObjectModel;
//@property(readonly) NSManagedObjectContext *managedObjectContext;

@end
