//
//  GLArrayView.h
//  GPSPhotoLinker
//
//  Created by Jeffrey Early on 12/29/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

/************************************************/
/*		GLArrayView								*/
/************************************************/

#pragma mark -
#pragma mark GLArrayView
#pragma mark

// For now this requires that you both bind it and set its datasource.

@class GLArrayViewItem;
@interface GLArrayView : NSView <NSDraggingSource,NSDraggingDestination,NSPasteboardItemDataProvider>

@property(strong) NSMutableArray *arrayItems;
@property(strong) id content;
@property(nonatomic, strong) id dataSource;
@property(copy) NSString * keyPath;
@property(strong) GLArrayViewItem *dragItem;
@property(strong) id dragInfo, candidateDragItem;
@property NSInteger dragItemIndex;
@property CGFloat leftMargin;
@property CGFloat rightMargin;
@property CGFloat topMargin;
@property CGFloat bottomMargin;
@property CGFloat nodeSpacing;
@property BOOL strictHeight; // whether or not to require the view to match the height of the sum of its nodes, default yes.
@property NSMutableArray *horizontalConstraints;
@property NSMutableArray *verticalConstraints; // There's one more vertical constraint than there is arrayItems.

-(void) addArrayViewItems: (NSArray *) anArray atIndexes: (NSIndexSet *) indexes;
-(void) removeArrayViewItemsAtIndexes: (NSIndexSet *) indexes;
-(NSInteger) indexOfPromisedDropLocation: (id <NSDraggingInfo>)sender;
- (void) reloadData;

- (NSSize) targetSizeAfterLayoutWithAnimation;
- (void) setWantsResizeLayoutWithAnimation: (BOOL) animate;

@end

/************************************************/
/*		GLArrayViewItem							*/
/************************************************/

#pragma mark -
#pragma mark GLArrayViewItem
#pragma mark

// This is the prototype ArrayView object *controller*. It acts as the mediator between
// the model objects in the array and their appropriate views.

@interface GLArrayViewItem : NSViewController

@property(strong) GLArrayView *arrayView;
@property(copy) NSString *dragType;

- (void) didAddViewToArrayView;
- (void) didRemoveViewFromArrayView;

@end

/************************************************/
/*		GLArrayViewDatasource					*/
/************************************************/

#pragma mark -
#pragma mark GLArrayViewDatasource
#pragma mark

@interface NSObject (GLArrayViewDatasource)

// Alternative to binding. This takes precedence.
- (NSUInteger) numberOfObjectsInArrayView: (GLArrayView *) arrayView;
- (id) arrayView: (GLArrayView *) arrayView objectAtIndex: (NSUInteger) index;

- (GLArrayViewItem *) arrayView: (GLArrayView *) arrayView itemForObject: (id) object;

// An array of strings -- the drag types that the view should register for.
- (NSArray *) dragTypesForArrayView: (GLArrayView *) arrayView;

// A draggable object can be picked up from the Array View,
// A moveable object will vacate its spot when picked up,
// An addable object can be added, and will create a new spot for the object.
// If the source and the desination are the same, then moveable and addable mean
// the same thing -- but in general it isn't true.
- (BOOL) arrayView: (GLArrayView *) arrayView canDragItem: (GLArrayViewItem *) item;
- (BOOL) arrayView: (GLArrayView *) arrayView canMoveItem: (GLArrayViewItem *) item;
- (BOOL) arrayView: (GLArrayView *) arrayView canAddItem: (GLArrayViewItem *) item;

// These are called to the DataSource once the user has completed the operation.
- (void) arrayView: (GLArrayView *) arrayView removeObject: (id) object;
- (void) arrayView: (GLArrayView *) arrayView addObject: (id) object atIndex: (NSUInteger) index;

@end